@@ -77,19 +77,21 @@ void CryptMsg::init(Crypt *ctx) throw(CSPException)
 
 }
 
-CryptMsg::CryptMsg(BYTE *STRING, DWORD LENGTH, Crypt *ctx) throw(CSPException)
+CryptMsg::CryptMsg(BYTE *STRING, DWORD LENGTH, Crypt *ctx, DWORD chunk_size) throw(CSPException)
 {
     LOG("CryptMsg::CryptMsg(%p, %lu, %p)\n", STRING, LENGTH, ctx);
     init(ctx);
     data = new BYTE[LENGTH];
     data_length = LENGTH;
+    data_chunk_size = chunk_size;
     memcpy(data, STRING, LENGTH);
 };
 
-CryptMsg::CryptMsg(Crypt *ctx) throw(CSPException)
+CryptMsg::CryptMsg(Crypt *ctx, DWORD chunk_size) throw(CSPException)
 {
     LOG("CryptMsg::CryptMsg(%p)\n", ctx);
     init(ctx);
+    data_chunk_size = chunk_size;
 }
 
 void CryptMsg::add_recipient(Cert *c) throw(CSPException)
@@ -495,9 +497,20 @@ void CryptMsg::encrypt_streaming(BYTE **s, DWORD *slen) throw(CSPException)
         throw CSPException("CryptMsg.encrypt_data_streaming: CryptMsgOpenToEncode failed.", err);
     }
     LOG("   opened message to encrypt\n");
-    if ( !CryptMsgUpdate(hmsg, data, data_length, TRUE) ) {
-        delete[] ei.rgpRecipients;
-        throw CSPException("CCryptMsg.encrypt_data_streaming: couldn't update message");
+    DWORD processed_bytes = 0;
+    DWORD chunk_size = 0;
+    for (BYTE *dataptr = data; processed_bytes < data_length; dataptr += data_chunk_size) {
+        chunk_size = data_chunk_size;
+        BOOL final_chunk = FALSE;
+        if (processed_bytes + chunk_size > data_length) {
+            chunk_size = data_length - processed_bytes;
+            final_chunk = TRUE;
+        }
+        if ( !CryptMsgUpdate(hmsg, dataptr, chunk_size, final_chunk) ) {
+            delete[] ei.rgpRecipients;
+            throw CSPException("CCryptMsg.encrypt_data_streaming: couldn't update message");
+        }
+        processed_bytes += chunk_size;
     }
     LOG("   updated message\n");
     if (!CryptMsgClose(hmsg)) {
@@ -579,12 +592,24 @@ void CryptMsg::sign_data_streaming(BYTE *STRING, DWORD LENGTH, BYTE **s, DWORD *
         throw CSPException("CryptMsg.sign_data_streaming: CryptMsgOpenToEncode failed.", err);
     }
     LOG("   opened message to encrypt\n");
-    printf("*** %i\n", data_length);
-    if ( !CryptMsgUpdate(hmsg, STRING, LENGTH, TRUE) ) {
-        delete[] ei.rgCertEncoded;
-        delete[] ei.rgSigners;
-        throw CSPException("CCryptMsg.sign_data_streaming: couldn't update message");
+
+    DWORD processed_bytes = 0;
+    DWORD chunk_size = 0;
+    for (BYTE *dataptr = STRING; processed_bytes < LENGTH; dataptr += data_chunk_size) {
+        chunk_size = data_chunk_size;
+        BOOL final_chunk = FALSE;
+        if (processed_bytes + chunk_size > LENGTH) {
+            chunk_size = LENGTH - processed_bytes;
+            final_chunk = TRUE;
+        }
+        if ( !CryptMsgUpdate(hmsg, dataptr, chunk_size, final_chunk) ) {
+            delete[] ei.rgCertEncoded;
+            delete[] ei.rgSigners;
+            throw CSPException("CCryptMsg.sign_data_streaming: couldn't update message");
+        }
+        processed_bytes += chunk_size;
     }
+
     if (!CryptMsgClose(hmsg)) {
         delete[] ei.rgCertEncoded;
         delete[] ei.rgSigners;
